
require 'open3'
require 'taeval/hadolint/config'

module Taeval
  module Hadolint
    class Runner
      def initialize(config, output, reporter)
        @config   = Config.new(config)
        @output   = output
        @reporter = reporter
      end

      def run
        solutions = Dir.glob("#{@config.solutions}/*")
        solutions.each do |repo|
          cmd = "docker run --rm -i -v #{@config.path}:/root/.config/hadolint.yaml hadolint/hadolint < #{repo}/docker/Dockerfile"
          stdout, stderr, status = Open3.capture3(cmd)
          @output.print "hadolint #{repo}/Dockerfile", stdout, stderr

          if 0 != status
            @reporter.add(repo: repo, runner: :hadolint, msg: "Status: #{status} Error: #{stderr}.") if !stderr.empty?
            @reporter.add(repo: repo, runner: :hadolint, msg: "Status: #{status} Message: #{stdout}.") if !stdout.empty? 
          end
        end
      end
    end
  end
end
